/****************************************************************************
 Module
   AnalogStuffSM.c

 Revision
   1.0.1

 Description
   This is a file for implementing state machines under the EF
   Event Framework.  This takes care of all of the analog stuff for
   the state machine for our tic tac doh board. 

 Notes
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "misc.h"
#include "EF_General.h"
#include "EF_Queue.h"
#include "EF_PostList.h"
#include "EF_Events.h"
#include "AnalogStuffSM.h"
#include "TicTacDohSM.h"
#include "ADS12.h"
#include "ServoControl.h"
//#include "ServoLib.h"
#include <stdio.h>
#include "EF_Timers.h"
#include <ME218_C32.h>

/*----------------------------- Module Defines ----------------------------*/
#define QUEUE_SIZE 3

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/
static boolean ADCInit(void);
static short checkAccelX1(void);
static short checkAccelY1(void);
static short checkAccelZ1(void);
static short checkPot1(void);
static short checkAccelX2(void);
static short checkAccelY2(void);
static short checkAccelZ2(void);
static short checkPot2(void);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static TicTacDohState_t CurrentState;

static long ShakeScore = 0;
static unsigned int CoinTossStartTime;
static const unsigned int MaxCoinTossTime = 3000; //A little more than 3 seconds
static const unsigned int MaxAimTime = 16000;
static const unsigned int TimeDecrement = 2000; //Number of ms between turning off LEDs for timer
static unsigned int TimeRemainingP1;
static unsigned int TimeRemainingP2;
static unsigned int LastTimeThresholdP1;
static unsigned int Dummy[4];
static unsigned int LastTimeThresholdP2;
static unsigned int StartAimTimeP1;
static unsigned int StartAimTimeP2;
static unsigned int StartVib1Time;
static unsigned int StartVib2Time;
static unsigned int MaxVibTime = 1000;
static boolean StopAimTimeP1;
static boolean StopAimTimeP2;

short ShakeAmpP1;
short ShakeAmpP2;
static short lastShakeAmpP1 = 173;
static short lastShakeAmpP2 = 173;
static short ShakeThreshold = 250;  // ADC reading that needs to be surpassed to be considered a valid shake
static short ShakeCountP1 = 0;
static short ShakeCountP2 = 0;
static short zeroGs = 173;  // ADC reading from accelerometer when no tilt/shake

static unsigned int VictoryDanceStartTime;
static unsigned int MaxVictoryDanceTime = 3000;    //Toggle LEDs for 3 seconds
static unsigned int LastVictoryDanceThreshold = 3000;
static unsigned int VictoryDanceDecrement = 300; //Toggle states of LEDs every 1/3 second

// everybody needs a queue too!
static EF_Event AnalogStuffSMQueue[QUEUE_SIZE+1];


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitAnalogStuffSM

 Parameters
     None

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Sets up the event queue and does any rother equired initialization for
     this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
boolean InitAnalogStuffSM ( void )
{
  unsigned char ucResult1;
  EF_Event ThisEvent;

Dummy[3] = 5;
  ucResult1 = EF_InitQueue( AnalogStuffSMQueue, ARRAY_SIZE(AnalogStuffSMQueue) );
  if ( ucResult1 ==  QUEUE_SIZE ) // successful queue init
  {
    // put us into the Initial PseudoState
    CurrentState = InitPState;
    // post the initial transition event
    ThisEvent.EventType = EF_INIT;
    if (PostAnalogStuffSM( ThisEvent ) == True)
    {
        return True;
    }else
    {
        return False;
    }
  }else
  {
    return False;
  }
}

/****************************************************************************
 Function
     PostAnalogStuffSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to the state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
boolean PostAnalogStuffSM( EF_Event ThisEvent )
{
  boolean bResult3;

  bResult3 = EF_EnQueueFIFO( AnalogStuffSMQueue, ThisEvent );

  return bResult3;
}

/****************************************************************************
 Function
    RunAnalogStuffSM

 Parameters
   void

 Returns
   boolean, Flase if an error encountered, True otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 10/23/11, 18:38
****************************************************************************/
boolean RunAnalogStuffSM( void )
{
  EF_Event ThisEvent;

  // pull an event from the queue
  ThisEvent = EF_DeQueue( AnalogStuffSMQueue );

  // we should only call the Run function if we know that there is something in
  // the queue, so if we found EF_NO_EVENT something is wrong
  if (ThisEvent.EventType == EF_NO_EVENT )
  {
    return False;
  }else
  { CurrentState = QueryTicTacDohSM();
    switch ( CurrentState )
    {
        case InitPState :       // If current state is initial Psedudo State
            if ( ThisEvent.EventType == EF_INIT )// only respond to EF_Init
            {
               ADCInit(); //Initialize ADC
               DDRT |= BIT3HI; //Set up pin T3 to be an output
               puts("ADC Initialized");
               if (Servo12_Init("xxxxxSSS") == SERVO12_OK)   
                 {
                    puts("ADC And Servos initialized Successfully");
                 }
                 else 
                 {
                    puts("Servo Initialzation Failed");
                 }
             }
             break;

        case WaitingToStart : 
            PTT &= BIT3LO; // Turn off vibration motor 
            LastVictoryDanceThreshold = MaxVictoryDanceTime; //Reset victory dance threshold timer
            TimeRemainingP1 = TimeDecrement;
            TimeRemainingP2 = TimeDecrement;
            LastTimeThresholdP1 = MaxAimTime;
            LastTimeThresholdP2 = MaxAimTime;
            if (LastTimeThresholdP2 != 16000)
            {
               puts("Counting Error in Waiting to Start\r\n");
            }
            StopAimTimeP1 = False;
            StopAimTimeP2 = False;
            CoinTossStartTime = EF_Timer_GetTime();   //Keep updating time                  
            ShakeCountP1 = 0;
            ShakeCountP2 = 0; //Reset to zero before starting shakeoff in coin toss
            break;       // end switch on CurrentEvent
        
        case CoinToss :   
            ShakeAmpP1 = checkAccelZ1();//Read both accelerometers
            ShakeAmpP2 = checkAccelZ2();
            //printf(" %d", ShakeAmpP1);
            //Keep track of each players shakes above threshold
            if ((ShakeAmpP1>ShakeThreshold) & (lastShakeAmpP1<=ShakeThreshold)) //This is the first reading of the shake to cross the threshold
		      {
		         ShakeCountP1 = ShakeCountP1 + 1; //Increment by one
		         printf("ShakeCountP1 = %d", ShakeCountP1);   
		      }
		      if ((ShakeAmpP2>ShakeThreshold) & (lastShakeAmpP2<=ShakeThreshold)) //This is the first reading of the shake to cross the threshold
		      {
		         ShakeCountP2 = ShakeCountP2 + 1; //Increment by one
		         printf("ShakeCountP2 = %d", ShakeCountP2);   
		      }
		      //Keep track of time within this state
		      if ((EF_Timer_GetTime()-CoinTossStartTime) > MaxCoinTossTime)  //If Timeout
            {
                if (ShakeCountP1 >= ShakeCountP2)  //Calculate who won the shake off
                { //Post event PlayerWinsToss with parameter 1 to Lists 00 02
                     EF_Event ThisEvent;
                     ThisEvent.EventType = EF_Player1WinsToss;
                     ThisEvent.EventParam = 1;
                     EF_PostList00(ThisEvent); // this could be any SM post function or EF_PostAll
                     EF_PostList02(ThisEvent);
                     puts("Player 1 Wins Toss");
                     StartAimTimeP1 = EF_Timer_GetTime(); //Start Timer for P1
                     StartVib1Time = EF_Timer_GetTime();
                } 
                else
                { //Post event PlayerWinsToss with parameter 2 to Lists 00 02
                     EF_Event ThisEvent;
                     ThisEvent.EventType = EF_Player2WinsToss;
                     ThisEvent.EventParam = 1;
                     EF_PostList00(ThisEvent); // this could be any SM post function or EF_PostAll
                     EF_PostList02(ThisEvent);
                     puts("Player 2 Wins Toss");
                     StartAimTimeP2 = EF_Timer_GetTime();  //StartTimer for P2
                     StartVib2Time = EF_Timer_GetTime();
                }     
            }     //End if
		      lastShakeAmpP1 = ShakeAmpP1; //Update for next time through state machine
		      lastShakeAmpP2 = ShakeAmpP2;
		      
		      TimeRemainingP1 = TimeDecrement;  //Set timers for Aiming States
            TimeRemainingP2 = TimeDecrement;
            LastTimeThresholdP1 = MaxAimTime;
            LastTimeThresholdP2 = MaxAimTime; 
            if (LastTimeThresholdP2 != 16000)
            {
               puts("Counting Error in Coin Toss\r\n");
            }
            StopAimTimeP1 = False; //Reset value to false
            StopAimTimeP2 = False; //Reset value to false
            break;
            
        case Player1Aiming :       
            if (StopAimTimeP2 == True)
            {
               TimeRemainingP2 = TimeRemainingP2 - (EF_Timer_GetTime()-StartAimTimeP2); //Update player 2's remaining time
            }
            if ((EF_Timer_GetTime() - StartVib1Time) < MaxVibTime)
            {
               PTT |= BIT3HI; // Turn on the vibration motor
            } else
            {
               PTT &= BIT3LO; // Turn off vibration motor 
            }
            StopAimTimeP2 = False; //Reset value to false
            AimGunX(checkAccelX1());//Read accelerometer of Player1 and Move servos to aim laser
            AimGunY(checkAccelY1());		
            SpinBoard(checkPot2());  //Read potentiometer of Player 2 and Move Board servo to match pot
         	//If Player 1 Time decrements past 2 sec interval
            if (((EF_Timer_GetTime()-StartAimTimeP1)) > TimeRemainingP1)	
            {
               EF_Event ThisEvent;
               ThisEvent.EventType = EF_Player1TimeDecrement;
               ThisEvent.EventParam = 1;
               EF_PostList02(ThisEvent); //Post event Player1TimeDecrement to list 02
               LastTimeThresholdP1 = LastTimeThresholdP1 - TimeDecrement; //Update value by subtracting interval
               TimeRemainingP1 = TimeDecrement;
               StartAimTimeP1 = EF_Timer_GetTime();
               printf("P1 Time Decrement Below = %d", LastTimeThresholdP1);
            }
            //If Player 1 Time all runs out
            //if ((TimeRemainingP1 - (int)(EF_Timer_GetTime()-StartAimTimeP1)) <= 0)
      		if (LastTimeThresholdP1 <= 0)
      		{
      		   EF_Event ThisEvent;
               ThisEvent.EventType = EF_Player1Timeout;
               ThisEvent.EventParam = 1;
               EF_PostList02(ThisEvent);
               EF_PostList00(ThisEvent); //Post Event PlayerTimeout with parameter 1 to lists 00 and 02
               LastTimeThresholdP1 = LastTimeThresholdP1 - TimeDecrement;
      		}      //End if           
            //Keep track of player 1's time remaining
            StartAimTimeP2 = EF_Timer_GetTime(); //StartTimer for P2
            StartVib2Time = EF_Timer_GetTime();
            StopAimTimeP1 = True; // Stop P1's timer when it switches to P2
            VictoryDanceStartTime = EF_Timer_GetTime();   //Keep updating time 
            break;       // end switch on CurrentEvent
            
        case Player2Aiming :       
            if (StopAimTimeP1 == True)
            {
               TimeRemainingP1 = TimeRemainingP1 - (EF_Timer_GetTime()-StartAimTimeP1);  //Update player1's time remaining
               printf("P2 Last Time Threshold in Transition = %u\n\r", LastTimeThresholdP2);
            }
            if ((EF_Timer_GetTime() - StartVib2Time) < MaxVibTime)
            {
               PTT |= BIT3HI; // Turn on the vibration motor
            } else
            {
               PTT &= BIT3LO; // Turn off vibration motor 
            }
            StopAimTimeP1 = False; //Reset value to false
            AimGunX(checkAccelX2());//Read accelerometer of Player2 and Move servos to aim laser
            AimGunY(checkAccelY2());		
            SpinBoard(checkPot1());  //Read potentiometer of Player 1 and Move Board servo to match pot
            //If Player 2 Time decrements past 2 sec interval
            if (((EF_Timer_GetTime()-StartAimTimeP2)) > TimeRemainingP2)
            {
               EF_Event ThisEvent;
               ThisEvent.EventType = EF_Player2TimeDecrement;
               ThisEvent.EventParam = 1;
               EF_PostList02(ThisEvent); //Post event Player1TimeDecrement to list 02
               LastTimeThresholdP2 = LastTimeThresholdP2 - TimeDecrement; //Update value by subtracting interval
               TimeRemainingP2 = TimeDecrement;
               StartAimTimeP2 = EF_Timer_GetTime();
               printf("P2 Time Decrement Below = %u", LastTimeThresholdP2);
            }
            //If Player 2 Time all runs out
            //if ((TimeRemainingP2 - (int)(EF_Timer_GetTime()-StartAimTimeP2)) <= 0)
      		if (LastTimeThresholdP2 <= 0)
      		{
      		   EF_Event ThisEvent;
               ThisEvent.EventType = EF_Player2Timeout;
               ThisEvent.EventParam = 1;
               EF_PostList02(ThisEvent);
               EF_PostList00(ThisEvent); //Post Event Player2Timeout to lists 00 and 02
               LastTimeThresholdP2 = LastTimeThresholdP2 - TimeDecrement;
      		}      //End if           
            //Keep track of player 1's time remaining
            StartAimTimeP1 = EF_Timer_GetTime(); //StartTimer for P1 before it switches back
            StartVib1Time = EF_Timer_GetTime();
            StopAimTimeP2 = True; // Stop P1's timer when it switches to P1
            VictoryDanceStartTime = EF_Timer_GetTime();   //Keep updating time 
            break;       // end switch on CurrentEvent
            
        case VictoryDance :       // If current state is state one
            //If victory dance decrements past a time interval
            PTT |= BIT3HI; // Turn on the vibration motor
            if ((LastVictoryDanceThreshold - (MaxVictoryDanceTime - (EF_Timer_GetTime()-VictoryDanceStartTime))) > VictoryDanceDecrement)		
            {
               EF_Event ThisEvent;
               ThisEvent.EventType = EF_VictoryDanceTimeDecrement;
               ThisEvent.EventParam = 1;
               EF_PostList02(ThisEvent); //Post event VictoryDanceTimeDecrement to list 02
               LastVictoryDanceThreshold = LastVictoryDanceThreshold - VictoryDanceDecrement; //Update value by subtracting interval
               printf("Victory Time Decrement Below = %u", LastVictoryDanceThreshold);
            }
            //If Victory Dance Time all runs out
            if (((EF_Timer_GetTime()-VictoryDanceStartTime)) > MaxVictoryDanceTime)
      		{
      		   EF_Event ThisEvent;
               ThisEvent.EventType = EF_VictoryDanceTimeout;
               ThisEvent.EventParam = 1;
               EF_PostList02(ThisEvent);
               EF_PostList00(ThisEvent); //Post Event VictoryDanceTimeout with parameter 1 to lists 00 and 02
      		}      //End if                         
            break;       // end switch on CurrentEvent

    }                                   // end switch on Current State
    return True;
  }
}

 /****************************************************************************
 Function
     IsAnalogStuffSMQEmpty

 Parameters
     void

 Returns
     boolean True if the Queue is empty, False if the queue has something in it

 Description
     tests if this state machine's queue is empty
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:28
****************************************************************************/
boolean IsAnalogStuffSMQEmpty( void ){

  return EF_IsQueueEmpty( AnalogStuffSMQueue );
}

/****************************************************************************
 Function
     QueryAnalogStuffSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
TicTacDohState_t QueryAnalogStuffSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

static boolean ADCInit(void)
{
if (ADS12_Init("AAAAAAAA") == ADS12_OK)
	{
	return True;
	}
else
	{
	puts("ADC Initialization Failed");
	return False;
	}
}

static short checkAccelX1(void)
{
	short AccelX1;
	AccelX1 = ADS12_ReadADPin(0);
	return AccelX1;
}

static short checkAccelY1(void)
{
	short AccelY1;
	AccelY1 = ADS12_ReadADPin(1);
	return AccelY1;
}

static short checkAccelZ1(void)
{
	short AccelZ1;
	AccelZ1 = ADS12_ReadADPin(2);
	return AccelZ1;
}

static short checkPot1(void)
{
	short Pot1;
	Pot1 = ADS12_ReadADPin(3);
	return Pot1;
}

static short checkAccelX2(void)
{
	short AccelX2;
	AccelX2 = ADS12_ReadADPin(4);
	return AccelX2;
}

static short checkAccelY2(void)
{
	short AccelY2;
	AccelY2 = ADS12_ReadADPin(5);
	return AccelY2;
}

static short checkAccelZ2(void)
{
	short AccelZ2;
	AccelZ2 = ADS12_ReadADPin(6);
	return AccelZ2;
}

static short checkPot2(void)
{
	short Pot2;
	Pot2 = ADS12_ReadADPin(7);
	return Pot2;
}